@@ -58,6 +58,12 @@ struct _GKMKeyringEditorPrivate
   GtkWidget *secret_field;
   GtkWidget *secret_multiline;
   GtkWidget *secret_scroll;
+  GtkWidget *secret_event_box;
+  GtkWidget *secret_expander;
+  GtkWidget *secret_dnd_label;
+  GtkWidget *delete_button;
+  GtkWidget *add_button;
+  GtkWidget *edit_button;
 
   gboolean is_multiline_secret;
 
@@ -133,7 +139,8 @@ static void gkm_keyring_editor_expander_changed_callback (GtkWidget *expander, G
 static void keyring_items_tree_context_menu_callback (GtkWidget *tree_view, GKMKeyringEditor *editor);
 static gboolean keyring_items_tree_button_press_callback (GtkWidget *tree_view, GdkEventButton *event, GKMKeyringEditor *editor);
 
-static void keyring_item_tree_selection_changed (GtkTreeView *tree_view, GKMKeyringEditor *editor);
+static void keyring_item_tree_selection_changed (GtkTreeSelection *selection, GKMKeyringEditor *editor);
+static void keyring_editor_update_sensitivity (GKMKeyringEditor *editor, gboolean sensitive);
 
 static void gkm_keyring_editor_update_keyring_items (GKMKeyringEditor *transient_parent);
 static void gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *editor, guint32 item_id, GtkTreeIter *iter);
@@ -247,10 +254,7 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   GtkWidget *frame;
   GtkWidget *scrolledwindow;
   GtkWidget *label;
-  GtkWidget *button;
-  GtkWidget *event_box;
   GtkWidget *image;
-  GtkWidget *expander;
   GtkWidget *menubar;
   GtkWidget *alignment;
   GtkSizeGroup *button_sizegroup;
@@ -261,6 +265,7 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   GdkPixbuf *keyring_pixbuf;
   GError *error;
   char *text;
+  GtkTreeSelection *item_tree_selection;
 
   editor->priv = g_new0 (GKMKeyringEditorPrivate, 1);
 
@@ -329,9 +334,9 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   gtk_container_add (GTK_CONTAINER (scrolledwindow), editor->priv->items_tree_view);
   g_object_set_data (G_OBJECT (editor), "keyring-items", editor->priv->items_tree_view);
   
-  //FIXME: this should probably need to connect to selection-changed on the selection of the tree view...
-  g_signal_connect (editor->priv->items_tree_view, "cursor-changed", 
-		    G_CALLBACK (keyring_item_tree_selection_changed), editor);
+  item_tree_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view));
+  g_signal_connect( item_tree_selection, "changed",
+                    G_CALLBACK (keyring_item_tree_selection_changed), editor);
 
   gkm_keyring_editor_setup_keyring_items_tree (editor);
   g_signal_connect (editor->priv->items_tree_view, "button-press-event",
@@ -366,15 +371,15 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox), GTK_BUTTONBOX_START);
   gtk_box_pack_start (GTK_BOX (hbox), vbuttonbox, FALSE, FALSE, 0);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
-  gtk_size_group_add_widget (button_sizegroup, button);
-  gtk_box_pack_start (GTK_BOX (vbuttonbox), button, FALSE, FALSE, 0);
-  g_signal_connect (button, "clicked", G_CALLBACK (remove_keyring_item_callback), editor);
+  editor->priv->delete_button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+  gtk_size_group_add_widget (button_sizegroup, editor->priv->delete_button);
+  gtk_box_pack_start (GTK_BOX (vbuttonbox), editor->priv->delete_button, FALSE, FALSE, 0);
+  g_signal_connect (editor->priv->delete_button, "clicked", G_CALLBACK (remove_keyring_item_callback), editor);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_NEW);
-  gtk_size_group_add_widget (button_sizegroup, button);
-  gtk_box_pack_start (GTK_BOX (vbuttonbox), button, FALSE, FALSE, 0);
-  g_signal_connect (button, "clicked", G_CALLBACK (add_keyring_item_callback), editor);
+  editor->priv->add_button = gtk_button_new_from_stock (GTK_STOCK_NEW);
+  gtk_size_group_add_widget (button_sizegroup, editor->priv->add_button);
+  gtk_box_pack_start (GTK_BOX (vbuttonbox), editor->priv->add_button, FALSE, FALSE, 0);
+  g_signal_connect (editor->priv->add_button, "clicked", G_CALLBACK (add_keyring_item_callback), editor);
 
   /* Item Attributes group */
   group_vbox = gtk_vbox_new (FALSE, 12);
@@ -403,21 +408,21 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   gtk_button_box_set_layout (GTK_BUTTON_BOX (vbuttonbox), GTK_BUTTONBOX_START);
   gtk_box_pack_start (GTK_BOX (hbox), vbuttonbox, FALSE, FALSE, 0);
 
-  button = gtk_button_new_with_mnemonic (_("_Edit"));
-  gtk_size_group_add_widget (button_sizegroup, button);
-  gtk_box_pack_start (GTK_BOX (vbuttonbox), button, FALSE, FALSE, 0);
-  g_signal_connect (button, "clicked", G_CALLBACK (edit_keyring_item_callback), editor);
+  editor->priv->edit_button = gtk_button_new_with_mnemonic (_("_Advanced"));
+  gtk_size_group_add_widget (button_sizegroup, editor->priv->edit_button);
+  gtk_box_pack_start (GTK_BOX (vbuttonbox), editor->priv->edit_button, FALSE, FALSE, 0);
+  g_signal_connect (editor->priv->edit_button, "clicked", G_CALLBACK (edit_keyring_item_callback), editor);
 
-  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), button, _("Edit the selected item."), NULL);
+  gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), editor->priv->edit_button, _("Edit the selected item."), NULL);
 
-  expander = gtk_expander_new_with_mnemonic (_("_Show Secret"));
-  gtk_expander_set_spacing (GTK_EXPANDER (expander), 6);
-  gtk_box_pack_start (GTK_BOX (groups_vbox), expander, FALSE, FALSE, 0);
-  g_signal_connect (G_OBJECT (expander), "notify::expanded", 
+  editor->priv->secret_expander = gtk_expander_new_with_mnemonic (_("_Show Secret"));
+  gtk_expander_set_spacing (GTK_EXPANDER (editor->priv->secret_expander), 6);
+  gtk_box_pack_start (GTK_BOX (groups_vbox), editor->priv->secret_expander, FALSE, FALSE, 0);
+  g_signal_connect (G_OBJECT (editor->priv->secret_expander), "notify::expanded", 
   		    G_CALLBACK (gkm_keyring_editor_expander_changed_callback), editor);
  
   hbox = gtk_hbox_new (FALSE, 12);
-  gtk_container_add (GTK_CONTAINER (expander), hbox);
+  gtk_container_add (GTK_CONTAINER (editor->priv->secret_expander), hbox);
   
   alignment = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 4, 0, 2, 0);
@@ -445,11 +450,11 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
 		     G_CALLBACK (keyring_item_secret_changed_callback), editor);
 
   hbox = gtk_hbox_new (FALSE, 12);
-  event_box = gtk_event_box_new ();
-  gtk_event_box_set_visible_window (GTK_EVENT_BOX (event_box), FALSE);
+  editor->priv->secret_event_box = gtk_event_box_new ();
+  gtk_event_box_set_visible_window (GTK_EVENT_BOX (editor->priv->secret_event_box), FALSE);
   frame = gtk_frame_new ("");
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  gtk_container_add (GTK_CONTAINER (event_box), frame);
+  gtk_container_add (GTK_CONTAINER (editor->priv->secret_event_box), frame);
   keyring_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					     "stock_keyring", 48, 0, &error);
   if (keyring_pixbuf != NULL)
@@ -462,15 +467,15 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
        image = gtk_image_new_from_stock (GTK_STOCK_DND, GTK_ICON_SIZE_DND);
     }
   gtk_container_add (GTK_CONTAINER (frame), image);
-  gtk_box_pack_start (GTK_BOX (hbox), event_box, FALSE, FALSE, 0);
-  label = gtk_label_new (_("You can copy the secret by dragging this image"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), editor->priv->secret_event_box, FALSE, FALSE, 0);
+  editor->priv->secret_dnd_label = gtk_label_new (_("You can copy the secret by dragging this image"));
+  gtk_box_pack_start (GTK_BOX (hbox), editor->priv->secret_dnd_label, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (groups_vbox), hbox, FALSE, FALSE, 0);
-  gtk_drag_source_set (GTK_WIDGET (event_box),
+  gtk_drag_source_set (GTK_WIDGET (editor->priv->secret_event_box),
                        GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
                        drag_types, G_N_ELEMENTS (drag_types),
                        GDK_ACTION_COPY);
-  g_signal_connect (G_OBJECT (event_box), "drag_data_get",
+  g_signal_connect (G_OBJECT (editor->priv->secret_event_box), "drag_data_get",
                     G_CALLBACK (drag_keyring_item_secret_data_get_callback), editor);
 
   gtk_drag_dest_set (GTK_WIDGET (editor),
@@ -480,6 +485,8 @@ gkm_keyring_editor_init (GKMKeyringEditor *editor)
   g_signal_connect (G_OBJECT (editor), "drag_data_received",
 		    G_CALLBACK (drop_keyring_item_data_recieved_callback), editor);
 
+  keyring_editor_update_sensitivity (editor, FALSE);
+
   gtk_widget_show_all (main_vbox);
 }
 
@@ -869,6 +876,7 @@ attributes_saved_callback (GnomeKeyringResult result,
 {
   GtkTreePath *path;
   GtkTreeIter iter;
+  GtkTreeSelection *selection;
 
   if (result != GNOME_KEYRING_RESULT_OK)
     {
@@ -888,7 +896,26 @@ attributes_saved_callback (GnomeKeyringResult result,
   gtk_tree_path_free (path);
 
   gkm_keyring_editor_update_keyring_item_info (data->editor, data->item_id, &iter);
-  keyring_item_tree_selection_changed (GTK_TREE_VIEW (data->editor->priv->items_tree_view), data->editor);
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data->editor->priv->items_tree_view));
+  keyring_item_tree_selection_changed (selection, data->editor);
+}
+
+static void
+acl_saved_callback (GnomeKeyringResult result,
+		    SaveItemCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      return;
+    }
+
+  if (!gtk_tree_row_reference_valid (data->row))
+    {
+      g_warning (_("A row disapeared while we were waiting for the data..."));
+
+      return;
+    }
 }
 
 void
@@ -900,15 +927,39 @@ save_attributes (SaveItemCallbackData *data)
 
   gnome_keyring_item_set_attributes (data->editor->priv->keyring_name, data->item_id, attribute_list,
 				     (GnomeKeyringOperationDoneCallback) attributes_saved_callback,
-				     data, (GDestroyNotify) save_item_callback_data_free);
+				     data, NULL);
 
   gnome_keyring_attribute_list_free (attribute_list);
 }
 
+void
+save_acl (SaveItemCallbackData *data)
+{
+  GList *acl;
+
+  acl = gkm_attribute_editor_get_acl (data->attribute_editor);
+
+  gnome_keyring_item_set_acl (data->editor->priv->keyring_name, data->item_id, acl,
+			      (GnomeKeyringOperationDoneCallback) acl_saved_callback,
+			      data, (GDestroyNotify) save_item_callback_data_free);
+
+  gnome_keyring_acl_free (acl);
+}
+
+
+
 static gboolean
 edit_keyring_item_closed_callback (GtkWidget *widget, GdkEvent *event G_GNUC_UNUSED, SaveItemCallbackData *data)
 {
-  save_attributes (data);
+  if (data->attribute_editor->attributes_modified == TRUE)
+    {
+      save_attributes (data);
+    }
+  if (data->attribute_editor->acl_modified == TRUE)
+    {
+      save_acl (data);
+    }
+
 
   gtk_widget_destroy (widget);
 
@@ -1193,6 +1244,7 @@ typedef struct _DeleteItemCallbackData
 {
   GKMKeyringEditor *editor;
   GtkTreeIter	    iter;
+  GtkTreeSelection  *selection;
   guint32	    item_id;
 } DeleteItemCallbackData;
 
@@ -1210,9 +1262,20 @@ delete_item_done_callback (GnomeKeyringResult        result,
     {
       complain_about_gnome_keyring_bad_result (GTK_WINDOW (data->editor), result);
       return;
-    } 
+    }
 
-  gtk_list_store_remove (data->editor->priv->keyring_items, &data->iter);
+  if (gtk_list_store_remove (data->editor->priv->keyring_items, &data->iter))
+    {
+      gtk_tree_selection_select_iter (data->selection, &data->iter);
+    }
+  else
+    {
+      if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (data->editor->priv->keyring_items), &data->iter))
+        {
+          gtk_tree_selection_select_iter (data->selection, &data->iter);
+        }
+    }
+  gkm_keyring_editor_set_secret (data->editor, "");
 }
 
 static void
@@ -1238,7 +1301,6 @@ remove_keyring_item_callback (GtkWidget        *button G_GNUC_UNUSED,
  		              GKMKeyringEditor *editor)
 {
   DeleteItemCallbackData *data;
-  GtkTreeSelection *selection;
   GtkTreeModel *model;
 
   g_return_if_fail (GKM_IS_KEYRING_EDITOR (editor));
@@ -1246,9 +1308,9 @@ remove_keyring_item_callback (GtkWidget        *button G_GNUC_UNUSED,
   data = g_new0 (DeleteItemCallbackData, 1);
   data->editor = editor;
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view));
+  data->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view));
   
-  if (gtk_tree_selection_get_selected (selection, &model, &data->iter))
+  if (gtk_tree_selection_get_selected (data->selection, &model, &data->iter))
     {
       GtkWidget *dialog;
       GtkWidget *label;
@@ -1444,31 +1506,33 @@ keyring_item_secret_changed_callback (GtkWidget *entry G_GNUC_UNUSED,
   g_return_val_if_fail (GKM_IS_KEYRING_EDITOR (editor), FALSE);
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view));
-  gtk_tree_selection_get_selected (selection, &model, &iter);
-  gtk_tree_model_get (model, &iter, ITEM_COLUMN_ID, &item_id, ITEM_COLUMN_SECRET, &secret, -1);
-
-  new_secret = gkm_keyring_editor_get_secret (editor);
-
-  if (strcmp (new_secret, secret) != 0)
+  
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-      ChangeSecretCallbackData *data;
+      gtk_tree_model_get (model, &iter, ITEM_COLUMN_ID, &item_id, ITEM_COLUMN_SECRET, &secret, -1);
+      new_secret = gkm_keyring_editor_get_secret (editor);
 
-      data = g_new0 (ChangeSecretCallbackData, 1);
-      data->editor = editor;
-      data->item_id = item_id;
-      data->old_secret = g_strdup (secret);
-      data->new_secret = new_secret;
-      data->iter = iter;
-      
-      gnome_keyring_item_get_info (editor->priv->keyring_name,
-      			           item_id,
-			           (GnomeKeyringOperationGetItemInfoCallback) change_secret_get_info_callback,
-			           data, 
-			           (GDestroyNotify) change_secret_callback_data_free); 
+      if (strcmp (new_secret, secret) != 0)
+        {
+        ChangeSecretCallbackData *data;
+
+        data = g_new0 (ChangeSecretCallbackData, 1);
+	data->editor = editor;
+	data->item_id = item_id;
+	data->old_secret = g_strdup (secret);
+	data->new_secret = new_secret;
+	data->iter = iter;
+	
+	gnome_keyring_item_get_info (editor->priv->keyring_name,
+				     item_id,
+				     (GnomeKeyringOperationGetItemInfoCallback) change_secret_get_info_callback,
+				     data, 
+				     (GDestroyNotify) change_secret_callback_data_free); 
+	}
+
+      g_free (secret);
     }
-
-  g_free (secret);
-
+  
   return FALSE;
 }
 
@@ -1571,36 +1635,51 @@ change_displayed_attributes_callback (GnomeKeyringResult             result,
 }
 
 static void
-keyring_item_tree_selection_changed (GtkTreeView       *tree_view,
+keyring_item_tree_selection_changed (GtkTreeSelection  *selection,
                                      GKMKeyringEditor  *editor)
 {
   GtkTreeModel     *model;
-  GtkTreeSelection *selection;
   GtkTreeIter       iter;
   char             *secret;
   guint32	    item_id;
   ItemTreeSelectionCallbackData *data;
 
   g_return_if_fail (GKM_IS_KEYRING_EDITOR (editor));
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      keyring_editor_update_sensitivity (editor, TRUE);
 
-  selection = gtk_tree_view_get_selection (tree_view);
-  
-  gtk_tree_selection_get_selected (selection, &model, &iter);
+      data =  g_new0 (ItemTreeSelectionCallbackData, 1);
+      data->editor = editor;
 
-  data =  g_new0 (ItemTreeSelectionCallbackData, 1);
-  data->editor = editor;
+      gtk_tree_model_get (model, &iter, ITEM_COLUMN_ID, &item_id, ITEM_COLUMN_TYPE, &data->item_type, ITEM_COLUMN_SECRET, &secret, -1);
 
-  gtk_tree_model_get (model, &iter, ITEM_COLUMN_ID, &item_id, ITEM_COLUMN_TYPE, &data->item_type, ITEM_COLUMN_SECRET, &secret, -1);
-  
-  editor->priv->is_multiline_secret =  (data->item_type == GNOME_KEYRING_ITEM_NOTE) ? TRUE : FALSE;
-  gkm_keyring_editor_show_secret_widget (editor);
-  gkm_keyring_editor_set_secret (editor, secret);
- 
-  gnome_keyring_item_get_attributes (editor->priv->keyring_name, 
-  				     item_id, 
+      editor->priv->is_multiline_secret =  (data->item_type == GNOME_KEYRING_ITEM_NOTE) ? TRUE : FALSE;
+      gkm_keyring_editor_show_secret_widget (editor);
+      gkm_keyring_editor_set_secret (editor, secret);
+
+      gnome_keyring_item_get_attributes (editor->priv->keyring_name, 
+				     item_id, 
 				     (GnomeKeyringOperationGetAttributesCallback) change_displayed_attributes_callback,
 				     data, g_free);
- }
+    }
+  else
+    {
+      keyring_editor_update_sensitivity (editor, FALSE);
+    }
+}
+
+static void
+keyring_editor_update_sensitivity (GKMKeyringEditor *editor, 
+                                   gboolean sensitive)
+{
+  gtk_widget_set_sensitive (editor->priv->delete_button, sensitive);
+  gtk_widget_set_sensitive (editor->priv->edit_button, sensitive);
+  gtk_widget_set_sensitive (editor->priv->secret_label, sensitive);
+  gtk_widget_set_sensitive (editor->priv->secret_event_box, sensitive);
+  gtk_widget_set_sensitive (editor->priv->secret_dnd_label, sensitive);
+  gtk_widget_set_sensitive (editor->priv->secret_expander, sensitive);
+}
 
 /********************************************************
  * Search for a keyring item
